#include "Camera2D.h"

namespace JEngine
{
	////////////////////// SCREENWIDTH AND SCREEN HEITH
	Camera2D::Camera2D() :
		_scale(1.0f), _position(0.0f, 0.0f), _cameraMatrix(1.0f), _orthoMatrix(1.0f), _needsMatrixUpdate(true), _screenWidth(500), _screenHeight(500)
	{
	}


	Camera2D::~Camera2D()
	{
	}

	void Camera2D::init(int screenWidth, int screenHeight)
	{
		_screenWidth = screenWidth;
		_screenHeight = screenHeight;

		// build ortho matrix
		// pos offset left, width, pos offset bottom, screenheight
		_orthoMatrix = glm::ortho(0.0f, (float)screenWidth, 0.0f, (float)screenHeight);
	}

	void Camera2D::update()
	{
		if (_needsMatrixUpdate)
		{
			glm::vec3 translate(-_position.x + _screenWidth / 2, -_position.y + _screenHeight / 2, 0.0f); // not 3D :P

			// convert to screen coords
			_cameraMatrix = glm::translate(_orthoMatrix, translate);
			glm::vec3 scale(_scale, _scale, 0.0f);
			_cameraMatrix = glm::scale(_cameraMatrix, scale);

			_needsMatrixUpdate = false;
		}
	}


	glm::vec2 Camera2D::convertScreenToWorld(glm::vec2 screenCoords)
	{
		// invert Y direction
		screenCoords.y = _screenHeight - screenCoords.y;

		// ZERO is center! Make it that way!
		screenCoords -= glm::vec2(_screenWidth / 2, _screenHeight / 2);
		
		// Then take the translation into account
		
		// move camera to the left, x coord is decreasing, so:
		screenCoords += _position;

		// zoom in --> coords smaller
		// zoom out --> coords greater
		screenCoords /= _scale;		

		// ORDER MATTERS!!
		return screenCoords;
	}
}