#pragma once

#include <JEngine/Window.h>
#include <JEngine/GLSLProgram.h>
#include <JEngine/Camera2D.h>
#include <JEngine/InputManager.h>
#include <JEngine/SpriteBatch.h>

#include "Level.h"
#include "Player.h"
#include "Potion.h"

class Zombie;

namespace sys = JEngine;

enum class GameState {
	PLAY,
	EXIT
};

class MainGame
{
public:
    MainGame();
    ~MainGame();

    /// Runs the game
    void run();

private:
    /// Initializes the core systems
    void initSystems();

    /// Initializes the shaders
    void initShaders();

    /// Main game loop for the program
    void gameLoop();

    /// Handles input processing
    void processInput();

    /// Renders the game
    void drawGame();

	void updateAgents();
	void updateBullets();
	void initLevel();
	void updatePotions();
	void checkVictory();



    /// Member Variables
    sys::Window _window; ///< The game window
	sys::GLSLProgram _textureProgram; ///< The shader program
	sys::InputManager _inputManager; ///< Handles input
	sys::Camera2D _camera; ///< Main Camera

	std::vector<Level*> _levels; // vector of all levels
	std::vector<Human*> _humans;
	std::vector<Zombie*> _zombies;
	std::vector<Bullet> _bullets;
	std::vector<Potion> _potions;
	Player* _player;

	JEngine::SpriteBatch _agentSpriteBatch;

	int _screenWidth, _screenHeight, _fps, _currentLevel;
	int _humansKilled, _zombiesKilled, _humansBecomeZombie;

	GameState _gameState;
};

