#include "Player.h"

#include "Gun.h"
#include "Potion.h"

#include <JEngine/InputManager.h>
#include <SDL/SDL.h>

#include <iostream>


Player::Player() :
_currentGun(-1)
{
}


Player::~Player()
{
}


void Player::addPotion(PotionEffect effect)
{
	switch (effect)
	{
	case PotionEffect::SPEED_UP:
		if (_speedUpTicks == 0)
			_speed *= 2;
		_speedUpTicks = POTION_TIME;
		std::cout << "Picked up a speed potion!" << std::endl;
		break;
	case PotionEffect::EXTRA_DAMAGE:
		_guns[_currentGun]->amplifyDamage();
		std::cout << "Picked up a damage potion for your " << _guns[_currentGun]->getName() << "!" << std::endl;
		break;
	case PotionEffect::FASTER_SHOOTING:
		_guns[_currentGun]->amplifyRate();
		std::cout << "Picked up a firerate potion for your " << _guns[_currentGun]->getName() << "!" << std::endl;
		break;
	}
}

void Player::addGun(Gun* gun)
{
	_guns.push_back(gun);

	if (_currentGun == -1)
	{
		_currentGun = 0;
	}
}

void Player::init(float speed, glm::vec2 pos, JEngine::InputManager* inputManager, 
	JEngine::Camera2D* camera, std::vector<Bullet>* bullets)
{
	_speed = speed;
	_pos = pos;
	_color.setColor(0, 0, 186);
	_inputManager = inputManager;
	_camera = camera;
	_bullets = bullets;
	_health = 100.0f;
}

void Player::update(const std::vector<std::string>& levelData,
	std::vector<Human*>& humans,
	std::vector<Zombie*>& zombies)
{
	// really small way to check if should stop
	if (_speedUpTicks > 0 && --_speedUpTicks == 0)
	{
		_speed /= 2;
	}

	if (_inputManager->isKeyDown(SDLK_d))
	{
		_pos.x += _speed;
	}
	else if (_inputManager->isKeyDown(SDLK_a))
	{
		_pos.x -= _speed;
	}

	if (_inputManager->isKeyDown(SDLK_w))
	{
		_pos.y += _speed;
	}
	else if (_inputManager->isKeyDown(SDLK_s))
	{
		_pos.y -= _speed;
	}

	if (_inputManager->isKeyDown(SDLK_1) && _guns.size() >= 0)
	{
		_currentGun = 0;
	}
	if (_inputManager->isKeyDown(SDLK_2) && _guns.size() >= 1)
	{
		_currentGun = 1;
	}
	if (_inputManager->isKeyDown(SDLK_3) && _guns.size() >= 2)
	{
		_currentGun = 2;
	}
	if (_inputManager->isKeyDown(SDLK_4) && _guns.size() >= 3)
	{
		_currentGun = 3;
	}

	if (_currentGun != -1)
	{
		glm::vec2 mouseCoords = _inputManager->getMouseCoords();
		mouseCoords = _camera->convertScreenToWorld(mouseCoords);
		glm::vec2 centerPos = _pos + glm::vec2(AGENT_RADIUS);

		glm::vec2 dir = glm::normalize(mouseCoords - centerPos);

		_guns[_currentGun]->update(_inputManager->isKeyDown(SDL_BUTTON_LEFT),
			centerPos,
			dir,
			*_bullets);
	}

	collideWithLevel(levelData);
}